using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace FM
{
    /// <summary>
    /// Trieda AiItemHolder predstavuje loisko pre vetky doplujce informcie.
    /// </summary>
    [Serializable] public class AiItemHolder : TreeNode
    {
        /// <summary>
        /// Obsahuje zoznamy doplujcich informcii.
        /// </summary>
        private AiItems aiItems;
        /// <summary>
        /// Vrti zoznamy doplujcich informcii.
        /// </summary>
        public AiItems AiItems
        {
            get { return aiItems; }
        }
        /// <summary>
        /// Kontruktor s parametrom vytvor nov objekt typu AiItemHolder, priom ho automaticky zarad do stromu pod svojho
        /// predka (parent) a pridradi mu ID.
        /// </summary>
        /// <param name="parent">Odkaz na predka, pod ktorho bude zaraden nov uzol.</param>
        /// <param name="ID">Identifiktor loiska.</param>
        public AiItemHolder(TreeNode parent, String ID)
            : base(parent, ID)
        {
            aiItems = new AiItems();
            SetID(ID);
        }
        /// <summary>
        /// Identifiktor tohto loiska doplujcich informcii.
        /// </summary>
        private String id;
        /// <summary>
        /// Vracia identifiktor tohto loiska doplujcich informcii.
        /// </summary>
        public String Id
        {
            get { return id; }
        }
        /// <summary>
        /// Kontruktor bez parametra vytvor nov objekt typu AiItemHolder.
        /// </summary>
        protected AiItemHolder()
            : base()
        {
            aiItems = new AiItems();
        }

        /// <summary>
        /// Testuje uniktnos identifiktorov v loisku.
        /// </summary>
        /// <param name="ID">Identifiktor pre otestovanie.</param>
        /// <returns>Vracia true ak je unikny, inak false.</returns>
        public bool IsIDUnique(String ID)
        {
            foreach (AiItemHolder tn in Children)
                if (tn.Id == ID)
                    return false;
            return true;
        }

        /// <summary>
        /// Nastav identifiktor v loisku, najskr vak nahrad nepovolen znaky znakom '_'.
        /// Ak je vstupn parameter je uniktny identifiktor, tak sa nastav,
        /// ak nie tak sa vygeneruje nov.
        /// </summary>
        /// <param name="ID">Identifiktor pre loisko.</param>
        public void SetID(String ID)
        {
            ID.Replace(' ', '_');
            ID.Replace('.', '_');

            if (IsIDUnique(ID))
            {
                id = ID;
            }
            else
            {
                id = GenerateUniqueId(this.Parent, "plz");
            }
            name = Id;
        }

        /// <summary>
        /// Vygeneruje nov identifiktor
        /// </summary>
        /// <param name="parent">Rodiovsk uzol stromu.</param>
        /// <param name="prefix">Predpona pre generovan identifiktor.</param>
        /// <returns></returns>
        public string GenerateUniqueId(TreeNode parent, string prefix)
        {
            return prefix + (parent.CountChildren() + 1).ToString();
        }

        /// <summary>
        /// Vrti potomka s rovnakm identifiktorom ako vstupn parameter. 
        /// </summary>
        /// <param name="Id">Identifiktor potomka.</param>
        /// <returns>Vracia loisko pre doplujce informcie.</returns>
        public AiItemHolder GetChild(String Id)
        {
            return GetChild(Id, null);
        }

        /// <summary>
        /// Vrti nasledujceho potomka s identifiktorom Id za vrcholom last.
        /// V prpade, e al potomok s identifiktorom Id za last nie je, 
        /// a vrchol last nasiel, vrti last(!).
        /// Ak nenasiel ani vrchol last, vrati null.
        /// </summary>
        /// <param name="Id">Identifiktor potomka.</param>
        /// <param name="last">Uruje pozciu od ktorej mme hada vrchol s Id.</param>
        /// <returns>Vrti nasledujci vrchol s Id za vrcholom last.</returns>
        protected AiItemHolder GetChild(String Id, AiItemHolder last)
        {
            bool foundLastNode = (last == null) ? true : false;
            foreach (AiItemHolder item in Children)
            {
                // zaistena transparentnost grupy
                if (item.GetType() == typeof(DGroup))
                {
                    AiItemHolder found = item.GetChild(Id, last);
                    if (found != null) return found;
                }
                // ak chceme najst cez bodkovy zapis aj grupu, nie je tato cast zabalena do 'else':
                if (item == last) foundLastNode = true;
                else if (foundLastNode && item.Id == Id) return item;
            }
            return (foundLastNode) ? last : null;
        }
        /// <summary>
        /// Funkcia na vpis loiska.
        /// </summary>
        /// <returns>Textov reazec s vpisom loiska</returns>
        override protected String Dump()
        {
            return (base.Dump() + "Additional information: " + aiItems.Serialize() + "\r\n");
        }

        /// <summary>
        /// Kopruje vrchol aj s doplujcimi informciami.
        /// </summary>
        /// <returns>Vrti novovytvoren vrchol.</returns>
        public override TreeNode Clone()
        {
            AiItemHolder copy = (AiItemHolder)base.Clone();
            copy.aiItems = aiItems.Clone();
            copy.id = id;

            return (TreeNode)copy;
        }
    }
}