using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace FM
{
    /// <summary>
    /// Trieda predstavuje zoznam doplujcich informacii.
    /// </summary>
    [Serializable]
    public class AiItems
    {
        
        /// <summary>
        /// Zoznam doplujcich informacii.
        /// </summary>
        public ArrayList aiItems;

        /// <summary>
        /// Kontruktor pre przdny zoznam doplujcich informacii.
        /// </summary>
        public AiItems()
        {
            aiItems = new ArrayList();
        }
        /// <summary>
        /// Funkcia naklonuje nov zoznam s polokami doplujcich informcii zo zadanho.
        /// </summary>
        /// <returns>Zoznam s polokami doplujcich informcii.</returns>
        public AiItems Clone()
        {
            AiItems copy = (AiItems)MemberwiseClone();
            copy.aiItems = new ArrayList();
            foreach (AiItem item in aiItems)
                copy.aiItems.Add(item.Clone());
            return copy;
        }
       /// <summary>
        /// Naimportuje vetky poloky zo ablny doplujcich informacii a ulo do zoznamu dopl. informacii.
       /// </summary>
        /// <param name="template">ablna doplujcich informacii.</param>
        public void ImportTemplateArray(ArrayList template)
        {
            foreach (AiItem aiItem in template)
            {
                aiItems.Add(aiItem.Clone());
            }
        }
        /// <summary>
        /// Naimportuje vetky poloky zo ablny doplujcich informacii.
        /// </summary>
        /// <param name="template">ablna doplujcich informacii.</param>
        public void ImportAiItems(AiItems template)
        {
            aiItems = new ArrayList(); // pridane by TT
            ImportTemplateArray(template.aiItems);
        }
     
        /// <summary>
        /// Pridanie poloky s doplujcimi informciami do zoznamu aiItems.
        /// </summary>
        /// <param name="key">Kl poloky s doplujcimi informciami.</param>
        /// <param name="isSelectable">Uruje, i ide o textov(FALSE) alebo hodnotov(TRUE).</param>
        /// <param name="allPossibleValues">Referencia na truktru obsahujcu vetky mon nastaviten hodnoty.</param>
        public bool AddAi(String key, bool isSelectable, AllPossibleAiValues allPossibleValues)
        {
            AiItem aiItem = GetAi(key);
            if (aiItem == null)
            {
                AiItem newAiItem = new AiItem(key, isSelectable, allPossibleValues);
                aiItems.Add(newAiItem);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Vrti jednu poloku s doplujcimi informciami zo zoznamu doplujcich informcii
        /// charakterizovan klom.
        /// </summary>
        /// <param name="key">Kl poloky s doplujcimi informciami</param>
        /// <returns>Vracia poloku s doplujcimi informciami</returns>
        public AiItem GetAi(String key)
        {
            foreach (AiItem aiItem in aiItems)
            {
                if (aiItem.Key == key) 
                    return aiItem;
            }
            return null;
        }
        /// <summary>
        /// Odstrni poloku s doplujcimi informciami zo zoznamu doplujcich informcii
        /// charakterizovan klom.
        /// </summary>
        /// <param name="key">Poloka s doplujcimi informciami charakterizovan kom.</param>
        public void RemoveAi(String key)
        {
            AiItem aiItem = GetAi(key);
            if (aiItem != null)
            {
                aiItem.DeletePossibleAiValuesSet();
                aiItems.Remove(aiItem);
            }
        }
        
        /// <summary>
        /// Vpis poloiek s doplujcimi informciami zo zoznamu aiItems.
        /// </summary>
        /// <returns>Textov vpis.</returns>
        public String Serialize()
        {
            String ret = "";
            foreach (AiItem a in aiItems)
            {
                ret += "\r\n  " + a.Key + " = " + a.CurrentValue;
            }
            return ret;
        }
    }
}
