using System;
using System.Collections;
using System.Text;

namespace FM
{
    /// <summary>
    /// Trieda Constraints obsahuje obmedzenia diagramu.
    /// </summary>
    [Serializable]
    public class Constraints
    {

        /// <summary>
        /// Opertor negcie.
        /// </summary>
        private String my_neg = "-";
        
        /// <summary>
        /// Opertor logickho nsobenia.
        /// </summary>
        private String my_and = "&&";
        
        /// <summary>
        /// Opertor logickho stania.
        /// </summary>
        private String my_or = "||";
        
        /// <summary>
        /// Opertor ekvivalencie.
        /// </summary>
        private String my_eq = "<=>";

        /// <summary>
        /// Opertor implikcie.
        /// </summary>
        private String my_imp = "=>";

        /// <summary>
        /// Zoznam vetkch obmedzen diagramu.
        /// </summary>
        private ArrayList constraints;
        
        /// <summary>
        /// Premenn typu diagram ktorej sa prirad hodnota vzobnho diagramu.
        /// </summary>
        private Diagram diagram;

        /// <summary>
        /// Kontruktor triedy vytvor pole constraintov typu ArrayList
        /// </summary>
        /// <param name="diagram">Vstupnm parametrom Constraints je diagram, na ktor sa bud jednotliv obmedzenia vzahova</param>
        public Constraints(Diagram diagram)
        {
            this.diagram = diagram;
            constraints = new ArrayList();
        }

        /// <summary>
        /// Metda ValidateAll prejde postupne pole constraints a vyhodnot vetky vrazy.
        /// </summary>
        /// <param name="diagram">Vstupnm parametrom je uzol stromu (intancia) ktorej obmedzenia sa vyhodnocuj.</param>
        /// <returns>Nvratovou hodnotou je seln hodnota, ktor vypoved o vsledku vyhodotenia vrazu. </returns>
        public int ValidateAll(Diagram diagram)
        {
            Expression e = new Expression(GetNewid(), "", my_neg, my_and, my_or, my_eq, my_imp);
            
            String complet_constraint = "";
            
            
            int i = 0;
            for (i =0; i < constraints.Count; i++)
            {
                e = (Expression)constraints[i];
                if (i == constraints.Count - 1)
                {
                    complet_constraint += e.ParseExpression(diagram);
                }
                else 
                {
                    complet_constraint += e.ParseExpression(diagram) + my_and;
                }
            }
            complet_constraint = e.EvaluateSimpleExpression(complet_constraint);
            if (complet_constraint == "0")
            {
                // obmedzenie z podmienky 
                return 0;
            }
            if (complet_constraint == "1")
            {
                return 1;
            }
            if (complet_constraint == "2")
            {
                // nevieme rozhodnut alebo este moze prist 0
                return 2;
            }
            if (complet_constraint.Trim().StartsWith("3"))
            {
                return 3;
            }
            return 3;
        }

        /// <summary>
        /// Public metda AddConstraint m za lohu na zklade prijatho textu vytvori obmedzenie a prida ho medzi constraints.
        /// </summary>
        /// <param name="expression">Vstupn reazec pre obmedzenie ktor sa bude vyhodnocova.</param>
        public void AddConstraint(String expression)
        {
            Expression e = new Expression(GetNewid(), expression, my_neg,my_and,my_or,my_eq,my_imp);

            constraints.Add(e);
        }

        /// <summary>
        /// Public metoda RemoveConstraint odstrni obmedzenie poda identifikanho sla
        /// </summary>
        /// <param name="constraint_id">Je identifikan slo obmedzenia</param>
        /// <returns>Pokia bolo odstrnenie spen vrti sa true, inak sa vrti false.</returns>
        public bool RemoveConstraintAt(int constraint_id)
        {
            foreach (Expression e in constraints)
            {
                if (e.GetID == constraint_id)
                {
                    constraints.Remove(e);
                    return true;
                }
            }
            return false;
        }


        /// <summary>
        /// Public metda odstrni prv obmedzenie rovnakho obsahu ako je vstup metdy
        /// </summary>
        /// <param name="constraint">Vstupn reazec obsahujci telo obmedzenia</param>
        /// <returns>Pokia bolo odstrnenie spen vrti sa true, inak sa vrti false.</returns>
        public bool RemoveConstraint(String constraint)
        {
            foreach (Expression e in constraints)
            {
                if (e.exp == constraint)
                {
                    constraints.Remove(e);
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Public metda GetConstraint vrti text konkrtneho obmedzenia na zklade id
        /// </summary>
        /// <param name="id">Vstupn parameter identifikuje obmedzenie ktor sa m obrazi</param>
        /// <returns>Nvratov hodnota je text obmedzenia</returns>
        public String GetConstraint(int id)
        {
            foreach (Expression e in constraints)
            {
                if (e.GetID == id)
                {
                    return e.exp;
                }
            }
            return null;
        }

        /// <summary>
        /// Public metda GetAllConstraints vrti vetky obmedzenia.
        /// </summary>
        /// <returns>Nvratov hodnota metdy je ArrayList.</returns>
        public ArrayList GetAllConstraints()
        {
            return constraints;
        }

        /// <summary>
        /// Privatna metda GetNewid m na starosti prideoanie identifikanch sel id novm constraintom.
        /// </summary>
        /// <returns>Vracia sa id novho constraintu.</returns>
        private int GetNewid()
        {
            int id = 0;
            return id;
        }

        /// <summary>
        /// Vytvra kpiu objektu.
        /// </summary>
        /// <param name="diagram">Diagram na ktor bude kpia naviazan.</param>
        /// <returns>Vrti vytvoren kpiu.</returns>
        public Constraints Clone(Diagram diagram)
        {
            Constraints copy = (Constraints)this.MemberwiseClone();
            copy.diagram = diagram;
            copy.constraints = new ArrayList();
            foreach (Expression item in constraints)
                copy.constraints.Add(item.Clone());

            return copy;
        }
    }
}
