using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace FM
{
    /// <summary>
    /// Trieda diagramu reprezentuje koncept alebo intanciu konceptu. Zapzdruje taktie obmedzenia, ablny doplujcich informci a DefaultDependencyRules, ktor sa viau priamo k diagramu.
    /// </summary>
    [Serializable]
    public class Diagram : DNode
    {
		/// <summary>
		/// Pouva sa pri validci a nastavuje sa ak diagram neme by za iadnych okolnost validn
		/// ani v budcich asoch viazania (s nevalidn niektor features, ktor maj as viazania men ako
		/// as intancicie).
		/// </summary>
		private bool canBeValid;
		/// <summary>
		/// Pouva sa pri validci a nastavuje sa ak diagram neme by za iadnych okolnost validn
		/// ani v budcich asoch viazania (s nevalidn niektor features, ktor maj as viazania men ako
		/// as intancicie).
		/// </summary>
		public bool CanBeValid
		{
			get
			{
				return canBeValid;
			}
			set
			{
				canBeValid = value;
			}
		}
        /// <summary>
        /// Referencia na intanciu abln doplujcich informci v projekte.
        /// </summary>
        public NodeAiTemplates NodeAiTemplates
        {
            get { return Project.NodeAiTemplates; }
            //set { Project.NodeAiTemplates = value; }
        }

        /// <summary>
        /// Dr odkaz na default dependencies rules.
        /// </summary>
        private DefaultDependencyRules defaultDependencyRules;
        /// <summary>
        /// Referencia na intanciu default dependency rules.
        /// </summary>
        public DefaultDependencyRules DefaultDependencyRules
        {
            get { return defaultDependencyRules; }
        }

        /// <summary>
        /// Referencia na intanciu obmedzen.
        /// </summary>
        private Constraints constraints;
        /// <summary>
        /// Referencia na intanciu obmedzen.
        /// </summary>
        public Constraints Constraints
        {
            get { return constraints; }
        }

        /// <summary>
        /// Indiktor, i je diagram intanciou.
        /// </summary>
        private bool isInstance;
        /// <summary>
        /// Vracia true v prpade, e intancia triedy reprezentuje intanciu v projekte.
        /// </summary>
        public bool IsInstance
        {
            get { return isInstance; }
        }

        /// <summary>
        /// Inicializuje intanciu diagramu.
        /// </summary>
        /// <param name="parent">Referencia na projekt, v ktorom sa diagram bude nachdza.</param>
        /// <param name="isInstance">V prpade, e sa jedn o reprezentciu intancie konceptu je nadstaven na true.</param>
        /// <param name="ID">Identifiktor pre diagram</param>
        public Diagram(DNode parent, bool isInstance, String ID)
            : base(parent, ID)
        {
            defaultDependencyRules = new DefaultDependencyRules(this);
            constraints = new Constraints(this);

            this.isInstance = isInstance;

            this.AiItems.ImportAiItems(Project.NodeAiTemplates.DiagramAiItems);
        }

        /// <summary>
        /// Naimportuje AI ablnu.
        /// </summary>
        public void ImportAiTemplate()
        {
            //Project.NodeAiTemplates.DiagramAiItems = null;
            this.AiItems.ImportAiItems(Project.NodeAiTemplates.DiagramAiItems);
        }

        /// <summary>
        /// Prid nov diagram do projektu pod aktulny diagram pod zadanm indentifiktorom.
        /// </summary>
        /// <param name="ID">Unikatny identifiktor diagramu.</param>
        public Diagram AddDiagram(String ID)
        {
            Diagram d = new Diagram(this, false, ID);
            return d;
        }

        /// <summary>
        /// Vracia true ke sa jedn on intanciu reprezentujcu koncept.
        /// </summary>
        /// <returns>Boolean</returns>
        public bool isConcept()
        {
            return !IsInstance;
        }

        /// <summary>
        /// Vrti aktulnu hodnotu intancicie pre tto intanciu.
        /// </summary>
        public String InstantiationTime
        {
            get
            {
                if (Project.BindingTime != null && AiItems.GetAi(this.Project.BindingTime.Key) != null)
                    return AiItems.GetAi(this.Project.BindingTime.Key).CurrentValue;
                //@@##debug semi - original
                //else return "";
                //                  changed
                else return "binding time";


            }
        }

        /// <summary>
        /// Kopruje vrchol aj s Template informciami.
        /// </summary>
        /// <returns>Vrti novovytvoren vrchol.</returns>
        public override TreeNode Clone()
        {
            Diagram copy = (Diagram)base.Clone();
//            copy.SetID(copy.Id + "_copy");
            copy.defaultDependencyRules = this.defaultDependencyRules.Clone(copy);
            copy.constraints = this.constraints.Clone(copy);
            return (TreeNode)copy;
        }

        /// <summary>
        /// Vytvor nov intanciu konceptu a naviae ju ako potomka pre tento diagram.
        /// </summary>
        /// <returns>Vrti kore intancie.</returns>
        public Diagram CreateInstance(string instTime)
        {
            Diagram copy = (Diagram)this.Clone(); // Mal by najst tu spravnu Clone() metodu cez virtual/override
            copy.isInstance = true;
            copy.AiItems.GetAi(this.NodeBindingTime.Key).SetAi(instTime);
            copy.name = copy.name + "Instance";

            foreach (TreeNode item in Children)
            {
                if (item.GetType() != typeof(Diagram)) copy.AddChild(item.CloneTree());
            }
            this.AddChild(copy); // uz ma ako parenta svoj nadradeny diagram

            // prejdi cely strom, vyhodnot Mandatory a nastav selected
            foreach (CardinalityHolder item in (System.Collections.ArrayList)copy.Children.Clone())
            {
                if ((item.GetType() == typeof(DFeature))
                 || (item.GetType() == typeof(DGroup)))
                    item.ResolveFeatureSelection();
            }
            return copy;
        }

        /// <summary>
        /// Skopruje cel diagram a naviae ho ako potomka pre tento diagram.
        /// </summary>
        /// <returns>Vrti kore novho diagramu.</returns>
        public Diagram CloneDiagram()
        {
            Diagram copy = (Diagram)this.Clone(); // Mal by najst tu spravnu Clone() metodu cez virtual/override
            copy.isInstance = false;
            copy.name = copy.name + "Copy";

            foreach (TreeNode item in Children)
            {
                if (item.GetType() != typeof(Diagram)) copy.AddChild(item.CloneTree());
            }
            this.AddChild(copy);
            return copy;
        }

        /// <summary>
        /// Zist, i je z tohto diagramu vytvoren nejak intancia.
        /// </summary>
        public bool HasInstance
        {
            get
            {
                foreach (DNode child in Children)
                {
                    if (child.GetType() == typeof(Diagram) && ((Diagram)child).IsInstance)
                    {
                        return true;
                    }
                }

                return false;
            }
        }
    }
}
