using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;



namespace FM
{/// <summary>
    /// Trieda Expression tvor obsah Constraints a DefaultDependencyRules
    /// </summary>
    [Serializable]
    public class Expression
    {
        /// <summary>
        /// Identifiktor vrazu.
        /// </summary>
        private int id;
        /// <summary>
        /// Textov obsah vrazu.
        /// </summary>
        private String expression;
        /// <summary>
        /// Opertor negcie.
        /// </summary>
        private String my_neg = "-";
        /// <summary>
        /// Opertor logickho nsobenia.
        /// </summary>
        private String my_and = "&&";
        /// <summary>
        /// Opertor logickho stania.
        /// </summary>
        private String my_or = "||";
        /// <summary>
        /// Opertor ekvivalencie.
        /// </summary>
        private String my_eq = "<=>";
        /// <summary>
        /// Opertor implikcie.
        /// </summary>
        private String my_imp = "=>";
        /// <summary>
        /// Nzov AiItem asu viazania.
        /// </summary>
        private String ITime_name = "";
        /// <summary>
        /// Pomocn premenn typu DFeature.
        /// </summary>
        private DFeature f;


        /// <summary>
        /// Kontruktor triedy Expression dostva 7 vstupnch parametrov a vytvra objkty tejto triedy
        /// </summary>
        /// <param name="id">Je jedinen slo v rmci Constraints, alebo DefaultDependencyRules</param>
        /// <param name="expression">Je reazec obsahujci text obmedzenia, alebo pravidla.</param>
        /// <param name="neg">Reazec oznaujci negciu. Defaultne "-".</param>
        /// <param name="and">Reazec oznaujci logick sin. Defaultne "&amp;&amp;".</param>
        /// <param name="or">Reazec oznaujci logick set. Defaultne "||".</param>
        /// <param name="eq">Reazec oznaujci ekvivalenciu.</param>
        /// <param name="imp">Reazec oznaujci implikciu. Defaultne "=>".</param>
        public Expression(int id, String expression, String neg, String and, String or, String eq, String imp)
        {
            this.id = id;
            this.expression = expression;
            my_neg = neg;
            my_and = and;
            my_or = or;
            my_eq = eq;
            my_imp = imp;
        }


        /// <summary>
        /// Manipuluje s obsahom vrazu.
        /// </summary>
        public String exp
        {
            get { return expression; }
            set { this.expression = value; }
        }

        /// <summary>
        /// Vracia identifikan slo vrazu
        /// </summary>
        public int GetID
        {
            get { return id; }
        }
        
       
        /// <summary>
        /// Public metda ParseExpession m za lohu vyhodnoti expression vzhadom na vstupn uzol stromu. 
        /// </summary>
        /// <param name="relative_root">Vstupnm parametrom je relatvny uzol stromu, vzhadom na ktor sa vyhodnot vraz</param>
        /// <returns>Nvratovou hodnotou je spracovan vraz</returns>
        public String ParseExpression(DNode relative_root)
        {
            AiItem cas_viazania_vlastnosti;
            AiItem cas_viazania_diagramu;
            AiItem binding_time;
            bool allow_continue = false ;

            if ((binding_time = relative_root.Project.BindingTime) == null)
            {
                return "3 Error: Not selected BINDING TIME additional information";
            }
            else if ((ITime_name = binding_time.Key) == "")
            {
                return "3 Error: Invalid BINDING TIME name";
            }               

            String exp = expression;

            int start_index = 0;
          //  int lenght = 0;

            string[] features = exp.Split(new string[] { my_neg, my_and, my_eq, my_imp, my_or, "(", ")" }, System.StringSplitOptions.RemoveEmptyEntries);

            for (int i = 0; i < features.Length; i++)
            {
                allow_continue = true;
                String s = features[i].Trim();
                if ((s != ""))
                {
                    start_index = exp.IndexOf(s);

                    if ((f = (DFeature)relative_root.GetNodeFromPath(s)) == null)
                    {
                        return "3 Error: Invalid Feature name";
                    }
                    else
                    {
                        if (f.Mandatory)
                        {
                            // povinna vlastnos
                            if (f.Selected)
                            {
                                exp = exp.Remove(start_index, s.Length);
                                exp = exp.Insert(start_index, "1");
                            }
                            else
                            {
                                exp = exp.Remove(start_index, s.Length);
                                exp = exp.Insert(start_index, "0");
                            }
                        }
                        else
                        {
                            if((cas_viazania_vlastnosti = f.AiItems.GetAi(ITime_name)) == null)
                            {
                                return ("3 Error: Invalid BINDING TIME additional information defined in " + f.Name);
                            }                            
                            

                            if (relative_root.GetType() == typeof(Diagram))
                            {
                                if ((cas_viazania_diagramu = relative_root.AiItems.GetAi(ITime_name)) == null)
                                {
                                    return ("3 Error: Invalid BINDING TIME additional information defined in " + relative_root.Name);
                                }
                            }
                            else
                            {
                                if ((cas_viazania_diagramu = relative_root.Diagram.AiItems.GetAi(ITime_name)) == null)
                                {
                                    return ("3 Error: Invalid BINDING TIME additional information defined in " + relative_root.Name);
                                }
                            }                            
                            
                            int tmp_order = -1;
                            if ((tmp_order = cas_viazania_vlastnosti.Order(cas_viazania_vlastnosti.CurrentValue, cas_viazania_diagramu.CurrentValue)) != -1)
                            {
                                if (tmp_order > 0)
                                {
                                    if (f.Selected)
                                    {
                                        exp = exp.Remove(start_index, s.Length);
                                        exp = exp.Insert(start_index, "1");
                                    }
                                    else
                                    {
                                        exp = exp.Remove(start_index, s.Length);
                                        exp = exp.Insert(start_index, "0");
                                    }
                                }
                                else
                                {
                                    exp = exp.Remove(start_index, s.Length);
                                    exp = exp.Insert(start_index, "2");
                                }
                            }
                            else 
                            {
                                return "3 Error: error occurred durring BINDING TIME comparation. ";
                            }

                        }
                    }
                }
            }
            if (allow_continue) return (this.EvaluateParsedExpression(exp));
            else return "3 Error: error input expression. ";

            /* ALGORITMUS 
                        while (end_index = exp_tmp.IndexOfAny(new char[] { my_and, my_or, my_eq, my_imp, ')' }) != -1)
                        {
                            if ((start_index = exp_tmp.LastIndexOfAny(new char[] { my_and, my_or, my_eq, my_imp, '(' }, 0, end_index)) != -1)
                            {
                                feature_name = exp_tmp.Substring(start_index + 1, end_index - 2).Trim();
                            }
                            else
                            {
                                //prvy objekt
                                feature_name = exp_tmp.Substring(0, end_index - 1).Trim();
                            }

                            //zisti ci je povinna vlastnost
                            // ano ==> is_selected?
                                       //ano ==> 1
                            /          /nie ==> 0
                            //nie ==> zisti ci je cas_viazania < binding_time
                                        //ano ==> is_selected?
                                            //ano  ==> 1
                                            //nie  ==> 0
                                        //nie ==> 2

                        }
            */
                        
            
        }

        /// <summary>
        /// Metda vyhodnot vstupn reazec do hbky (rekurzvne).
        /// </summary>
        /// <param name="exp">Vstupn reazec s vnranm cez ztvorky, operciami NEG, AND, OR, IMP, EQ hodnotami 0,1,2 a (, ).</param>
        /// <returns>Nvratov hodnota je upraven reazec s vyhodnotenm vnranm a operciami.</returns>
        public String EvaluateParsedExpression(String exp)
        {

            exp = exp.Trim();


            if (exp.Contains(")"))
            {
                int end_part = exp.IndexOf(")");
                String exp_tmp = exp.Substring(0, end_part);

                int start_part = exp_tmp.LastIndexOf("(");

                if (start_part == -1)
                {
                    return "3 Error: Invalid BRACKET pairs";
                }
                else
                {
                    exp = exp.Insert(start_part, EvaluateSimpleExpression(exp_tmp.Substring(start_part + 1, exp_tmp.Length - start_part - 1)));
                    return this.EvaluateParsedExpression(exp.Remove(start_part + 1, end_part - start_part + 1));
                }
            }
            else if (exp.Contains("("))
            {
                return "3 Error: Invalid BRACKET pairs";
            }
            else
            {
                return (EvaluateSimpleExpression(exp));
            }

        }

        /// <summary>
        /// Metoda vyhodnot jednoduch vstupn reazec (bez ztvoriek)
        /// </summary>
        /// <param name="exp">Vstupn reazec s operciami NEG, AND, OR, IMP, EQ a hodnotami 0,1,2.</param>
        /// <returns>Nvratov hodnota je upraven reazec s vyhodnotenmi operciami.</returns>
        public String EvaluateSimpleExpression(String exp)
        {
            exp = EvaluateNegation(exp, my_neg);
            if (exp.Trim().StartsWith("3")) return exp;
            exp = EvaluateAnd(exp, my_and);
            if (exp.Trim().StartsWith("3")) return exp;
            exp = EvaluateOr(exp, my_or);
            if (exp.Trim().StartsWith("3")) return exp;
            exp = EvaluateEq(exp, my_eq);
            if (exp.Trim().StartsWith("3")) return exp;
            exp = EvaluateImp(exp, my_imp);
            return exp;
        }

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu NEGCIA vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <returns>Metda vrti upraven reazec bez operacii NEGCIA.</returns>
        private String EvaluateNegation(String exp)
        {
            exp.Trim();
            int start_point = 0;

            while ((start_point = exp.LastIndexOf("-")) != -1)
            {
                String exp_tmp = exp.Substring(start_point + 1).Trim();
                if (exp_tmp.StartsWith("0"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "1" + exp_tmp.Substring(1));
                }
                else if (exp_tmp.StartsWith("1"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "0" + exp_tmp.Substring(1));
                }
                else if (exp_tmp.StartsWith("2"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "2" + exp_tmp.Substring(1));
                }
                else
                {
                    //MessageBox.Show("Vyskytol sa znak \"-\" na neocakavanom mieste (nesmie tu byt umiestneny)");
                    return "3 Error: Invalid NEGATION operators";
                }
            }
            return exp;

        }

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu NEGCIA vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <param name="own_negation">Uvateom zadefinovan reazec operandu NEGCIA.</param>
        /// <returns>Metda vrti upraven reazec bez operacii NEGCIA.</returns>
        private String EvaluateNegation(String exp, String own_negation)
        {

            exp.Trim();
            if (own_negation == "") own_negation = "-";
            int start_point = 0;

            while ((start_point = exp.LastIndexOf(own_negation)) != -1)
            {
                String exp_tmp = exp.Substring(start_point + own_negation.Length).Trim();
                if (exp_tmp.StartsWith("0"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "1" + exp_tmp.Substring(1));
                }
                else if (exp_tmp.StartsWith("1"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "0" + exp_tmp.Substring(1));
                }
                else if (exp_tmp.StartsWith("2"))
                {
                    exp = exp.Remove(start_point);
                    exp = exp.Insert(start_point, "2" + exp_tmp.Substring(1));
                }
                else
                {
                    //MessageBox.Show("Vyskytol sa znak \"-\" na neocakavanom mieste (nesmie tu byt umiestneny)");
                    return "3 Error: Invalid NEGATION operators";
                }
            }
            return exp;
        }


        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu AND vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <returns>Metda vrti upraven reazec bez operacii AND.</returns>
        private String EvaluateAnd(String exp)
        {/*
          *              0 and 0 => 0
          *              0 amd 1 => 0
          *              0 and 2 => 0
          *              
          *              1 and 0 => 0
          *              1 and 1 => 1
          *              1 and 2 => 2
          *
          *              2 and 0 => 0
          *              2 and 1 => 2
          *              2 and 2 => 2    
          */
            exp.Trim();
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf("&")) != -1)
            {
                exp_po = exp.Substring(start_point + 1).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "0") { c = "0"; }
                else if (b == "0") { c = "0"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "1") && (b == "1")) { c = "1"; }
                else
                {
                    //MessageBox.Show("Operacia AND bola volana na nekorektny znak");
                    return "3 Error: Invalid AND operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;

        }
        
        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu AND vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <param name="own_and">Uvateom zadefinovan  reazec operandu AND.</param>
        /// <returns>Metda vrti upraven reazec bez operacii AND.</returns>
        private String EvaluateAnd(String exp, String own_and)
        {/*
          *              0 and 0 => 0
          *              0 amd 1 => 0
          *              0 and 2 => 0
          *              
          *              1 and 0 => 0
          *              1 and 1 => 1
          *              1 and 2 => 2
          *
          *              2 and 0 => 0
          *              2 and 1 => 2
          *              2 and 2 => 2    
          */
            exp.Trim();
            if (own_and == "") own_and = "&";
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf(own_and)) != -1)
            {
                exp_po = exp.Substring(start_point + own_and.Length).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "0") { c = "0"; }
                else if (b == "0") { c = "0"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "1") && (b == "1")) { c = "1"; }
                else
                {
                    //MessageBox.Show("Operacia AND bola volana na nekorektny znak");
                    return "3 Error: Invalid AND operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }


        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu OR vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <returns>Metda vrti upraven reazec bez operacii OR.</returns>
        private String EvaluateOr(String exp)
        {/*
          *              0 or 0 => 0
          *              0 or 1 => 1
          *              0 or 2 => 2
          *
          *              1 or 0 => 1
          *              1 or 1 => 1
          *              1 or 2 => 1
          * 
          *              2 or 0 => 2
          *              2 or 1 => 1
          *              2 or 2 => 2              
          */
            exp.Trim();
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf("|")) != -1)
            {
                exp_po = exp.Substring(start_point + 1).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "1") { c = "1"; }
                else if (b == "1") { c = "1"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "0") && (b == "0")) { c = "0"; }
                else
                {
                    //MessageBox.Show("Operacia OR bola volana na nekorektny znak");
                    return "3 Error: Invalid OR operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }


        /// <summary>
        /// Metda vyhodnocuje vetky vskyty operandu OR vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <param name="own_or">Uvateom zadefinovan  reazec operandu OR.</param>
        /// <returns>Metda vrti upraven reazec bez operacii OR.</returns>
        private String EvaluateOr(String exp, String own_or)
        {/*
          *              0 or 0 => 0
          *              0 or 1 => 1
          *              0 or 2 => 2
          *
          *              1 or 0 => 1
          *              1 or 1 => 1
          *              1 or 2 => 1
          * 
          *              2 or 0 => 2
          *              2 or 1 => 1
          *              2 or 2 => 2              
          */
            exp.Trim();
            if (own_or == "") own_or = "|";
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf(own_or)) != -1)
            {
                exp_po = exp.Substring(start_point + own_or.Length).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "1") { c = "1"; }
                else if (b == "1") { c = "1"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "0") && (b == "0")) { c = "0"; }
                else
                {
                    //MessageBox.Show("Operacia OR bola volana na nekorektny znak");
                    return "3 Error: Invalid OR operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty ekvivalencie vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <returns>Metda vrti upraven reazec bez ekvivalencie.</returns>
        private String EvaluateEq(String exp)
        {/*
          *             0 <=> 0         => 1
          *             0 <=> 1         => 0
          *             0 <=> 2         => 2
          *
          *             1 <=> 0         => 0
          *             1 <=> 1         => 1
          *             1 <=> 2         => 2
          * 
          *             2 <=> 0         => 2
          *             2 <=> 1         => 0
          *             2 <=> 2         => 2              
          */
            exp.Trim();
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf("<=>")) != -1)
            {
                exp_po = exp.Substring(start_point + 2).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();


                if (b == "2") { c = "2"; };
                if (a == b) { c = a; }
                else if (b == "1") { c = "0"; }
                else if ((a == "1") && (b == "0")) { c = "0"; }
                else if ((a == "2") && (b == "0")) { c = "2"; }
                else
                {
                    //MessageBox.Show("Operacia EKVIVALENCIE bola volana na nekorektny znak");
                    return "3 Error: Invalid EQUIVALENCE operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty ekvivalencie vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <param name="own_equivalence">Uvateom zadefinovan  reazec kevivalencie.</param>
        /// <returns>Metda vrti upraven reazec bez ekvivalencie.</returns>
        private String EvaluateEq(String exp, String own_equivalence)
        {/*
          *             0 <=> 0         => 1
          *             0 <=> 1         => 0
          *             0 <=> 2         => 2
          *
          *             1 <=> 0         => 0
          *             1 <=> 1         => 1
          *             1 <=> 2         => 2
          * 
          *             2 <=> 0         => 2
          *             2 <=> 1         => 0
          *             2 <=> 2         => 2              
          */
            exp.Trim();
            if (own_equivalence == "") own_equivalence = "<=>";
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf(own_equivalence)) != -1)
            {
                exp_po = exp.Substring(start_point + own_equivalence.Length).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();


                if (b == "2") { c = "2"; }
                else if (a == b) { c = "1"; }
                else if (b == "1") { c = "0"; }
                else if ((a == "1") && (b == "0")) { c = "0"; }
                else if ((a == "2") && (b == "0")) { c = "2"; }
                else
                {
                    //MessageBox.Show("Operacia EKVIVALENCIE bola volana na nekorektny znak");
                    return "3 Error: Invalid EQUIVALENCE operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }
        

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty implikcie vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie.</param>
        /// <returns>Metda vrti upraven reazec bez implikci.</returns>
        private String EvaluateImp(String exp)
        {/*
          *                 0 => 0         => 1
          *                 0 => 1         => 1
          *                 0 => 2         => 1
          *
          *                 1 => 0         => 0
          *                 1 => 1         => 1
          *                 1 => 2         => 2
          * 
          *                 2 => 0         => 2
          *                 2 => 1         => 1
          *                 2 => 2         => 2              
          */
            exp.Trim();
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf("=>")) != -1)
            {
                exp_po = exp.Substring(start_point + 2).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "0") { c = "1"; }
                else if (b == "1") { c = "1"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "1") && (b == "0")) { c = "0"; }
                else if ((a == "2") && (b == "0")) { c = "2"; }
                else
                {
                    //MessageBox.Show("Operacia IMPLIKACIE bola volana na nekorektny znak");
                    return "3 Error: Invalid IMPLICATION operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }

        /// <summary>
        /// Metda vyhodnocuje vetky vskyty implikcie vo vstupnom reazci.
        /// </summary>
        /// <param name="exp">Vstupn reazec na vyhodnotenie. </param>
        /// <param name="own_implication">Uvateom zadefinovan  reazec implikcie. </param>
        /// <returns>Metda vrti upraven reazec bez implikci. </returns>
        private String EvaluateImp(String exp, String own_implication)
        {/*
          *                 0 => 0         => 1
          *                 0 => 1         => 1
          *                 0 => 2         => 1
          *
          *                 1 => 0         => 0
          *                 1 => 1         => 1
          *                 1 => 2         => 2
          * 
          *                 2 => 0         => 2
          *                 2 => 1         => 1
          *                 2 => 2         => 2              
          */
            exp.Trim();
            if (own_implication == "") own_implication = "=>";
            int start_point = 0;
            String exp_po = "";
            String exp_pred = "";
            String a = "";
            String b = "";
            String c = "";

            while ((start_point = exp.LastIndexOf(own_implication)) != -1)
            {
                exp_po = exp.Substring(start_point + own_implication.Length).Trim();
                exp_pred = exp.Substring(0, start_point).Trim();

                a = exp_pred.Substring(exp_pred.Length - 1, 1);
                exp_pred = exp_pred.Substring(0, exp_pred.Length - 1).Trim();
                b = exp_po.Substring(0, 1);
                exp_po = exp_po.Substring(1).Trim();

                if (a == "0") { c = "1"; }
                else if (b == "1") { c = "1"; }
                else if (a == "2") { c = "2"; }
                else if (b == "2") { c = "2"; }
                else if ((a == "1") && (b == "0")) { c = "0"; }
                else if ((a == "2") && (b == "0")) { c = "2"; }
                else
                {
                    //MessageBox.Show("Operacia IMPLIKACIE bola volana na nekorektny znak");
                    return "3 Error: Invalid IMPLICATION operators";
                }
                exp = exp_pred + c + exp_po;
            }
            return exp;
        }

        /// <summary>
        /// Koprovanie expression.
        /// </summary>
        /// <returns>Vrti kpiu objektu.</returns>
        public Expression Clone()
        {
            return (Expression)this.MemberwiseClone();
        }
    }
}
