using System;
using System.Collections.Generic;
using System.Text;

namespace FM
{
	/// <summary>
	/// Trieda uchovva informcie o kardinalite uzla. Reprezentuje len bezprostredne svisiace informcie o
	/// minimlnej a maximlnej, resp. nekonenej kardinalite, interpretciu tchto informci nechva na triedy v ktorch je
	/// agregovan (DFeature a DGroup).
	/// </summary>
    [Serializable]
    public class NodeCardinality
    {
        /// <summary>
        /// Informcia o minimlnej kardinalite.
        /// </summary>
        private int min;
        /// <summary>
        /// Nastavuje alebo zskava minimlnu kardinalitu.
        /// </summary>
        public int Min
        {
            get
            {
                return min;
            }
            set
            {
                min = value;
            }
        }

        /// <summary>
        /// Informcia o maximlnej kardinalite.
        /// </summary>
        private int max;
        /// <summary>
        /// Nastavuje resp. zskava maximlnu kardinalitu.
        /// Pri zskavan hodnoty: ak je infinite==true, vrati -1
        /// Pri nastavovani hodnoty: ak je hodnota mensia ako 0, nastav sa infinte na true;
        /// inak nastav infinite na false a max na novu hodnotu.
        /// </summary>
        public int Max
        {
            get
            {
                if (infinite == true)
                {
                    return -1;
                }
                else
                {
                    return max;
                }
            }
            set
            {
                if (value < 0)
                {
                    infinite = true;
                }
                else
                {
                    infinite = false;
                    if (value < min)
                    {
                        max = min;
                    }
                    else
                    {
                        max = value;
                    }
                }
            }
        }

        /// <summary>
        /// Informcia o tom, i je maximlna kardinalita nekonen.
        /// </summary>
        private bool infinite;
        /// <summary>
        /// Zskava, resp. nastavuje maximlnu kardinalitu na nekoneno.
        /// </summary>
        public bool Infinite
        {
            get
            {
                return infinite;
            }
            set
            {
                infinite = value;
            }
        }

		/// <summary>
		/// Jeden z kontruktorov. Implicitne nastav kardinalitu na 0..*
		/// </summary>
		public NodeCardinality()
		{
			this.SetCardinality(0);
		}

		/// <summary>
		/// Jeden z kontruktorov. Nastav kardinalitu na min..*
		/// </summary>
		/// <param name="min">Minimlna kardinalita</param>
		public NodeCardinality(int min)
		{
			this.SetCardinality(min);
		}

		/// <summary>
		/// Jeden z kontruktorov. Nastav kardinalitu na min..max
		/// </summary>
		/// <param name="min">Minimlna kardinalita</param>
		/// <param name="max">Maximlna kardinalita</param>
		public NodeCardinality(int min, int max)
		{
			this.SetCardinality(min, max);
		}


		/// <summary>
		/// Prepna maximlnu kardinalitu z nekonena na koneno (uren pomocou max) a naopak.
		/// </summary>
		/// <returns>Vracia nov hodnotu po prepnut.</returns>
		public bool ToggleInfinite()
		{
			this.infinite = !this.infinite;
			return infinite;
		}

		/// <summary>
		/// Nastavuje kardinalitu na min..*
		/// </summary>
		/// <param name="min">Minimlna kardinalita.</param>
		public void SetCardinality(int min)
		{
			this.Min = min;
			this.Infinite = true;
		}

		/// <summary>
		/// Nastavuje kardinalitu na min..max
		/// </summary>
		/// <param name="min">Minimlna kardinalita</param>
		/// <param name="max">Maximlna kardinalita</param>
		public void SetCardinality(int min, int max)
		{
			this.Min = min;
			this.Max = max;
		}

		/// <summary>
		/// Vracia retazec v tvare min..max )ak je maximlna kardinalita konen), resp. min..* (ak je maximlna
		/// kardinalita nekonen).
		/// </summary>
		public String Cardinality
		{
			get
			{
				String ret = "";
				ret += Min;
				ret += "..";
				if (Infinite == true)
				{
					ret += "*";
				}
				else
				{
					ret += Max;
				}
				return ret;
			}
		}

        /// <summary>
        /// Koprovanie objektu.
        /// </summary>
        /// <returns>Vrti kpiu objektu.</returns>
        public NodeCardinality Clone()
        {
            return (NodeCardinality)this.MemberwiseClone();
        }
    }
}
