using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace FM
{
    /// <summary>
    /// Trieda zapuzdruje cel projekt. Tto trieda sli ako orientan bod pre vetky ostatn subsystmy kede objahuje vinu referenci.
    /// </summary>
    [Serializable] 
    public class Project : DNode
    {
        private NodeAiTemplates nodeAiTemplates;
        /// <summary>
        /// Referencia na intanciu abln doplujcich informci, pre jednotliv uzly diagramu.
        /// </summary>
        public NodeAiTemplates NodeAiTemplates
        {
            get { return nodeAiTemplates; }
            set { nodeAiTemplates = value; }
        }

        /// <summary>
        /// Referencia na preddefinovan doplujce informcie pre uzol typu projekt.
        /// </summary>
        public AllPossibleAiValues aPVProject
        {
            get { return aPVProject; }
            set { aPVProject = value; }
        }
        /// <summary>
        /// Referencia na preddefinovan doplujce informcie pre uzol typu diagram.
        /// </summary>
        public AllPossibleAiValues aPVDiagram
        {
            get { return aPVDiagram; }
            set { aPVDiagram = value; }
        }
        /// <summary>
        /// Referencia na preddefinovan doplujce informcie pre uzol typu vlastnos.
        /// </summary>
        public AllPossibleAiValues aPVFeature
        {
            get { return aPVFeature; }
            set { aPVFeature = value; }
        }
       
        /// <summary>
        /// Udruje globlny as viazania v projekte
        /// </summary>
        private AiItem bindingTime;
        /// <summary>
        /// Odkaz na poloku AiItem, ktor obsahuje BindingTime pre Vlastnosti a InstatiationTime pre 
        /// Intancie.
        /// </summary>
        public AiItem BindingTime
        {
            get
            {
//                throw new System.NotImplementedException();
                return bindingTime;
            }
            set
            {
                bindingTime = value;
            }
        }

        /// <summary>
        /// Inicializuje instanciu projektu a ablnu doplujcich informci pre novovytvran diagramy.
        /// </summary>
        public Project()
			: base(null, "project")
		{
            nodeAiTemplates = new NodeAiTemplates();
        }

        /// <summary>
        /// Naimportuje pre projekt dodaton informcie z templatu.
        /// </summary>
        public void ImportAiTemplate()
        {
            this.AiItems.ImportAiItems(NodeAiTemplates.ProjectAiItems);
        }

        /// <summary>
        /// Prid nov diagram do projektu pod zadanm indentifiktorom.
        /// </summary>
        /// <param name="ID">Unikatny identifiktor diagramu.</param>
        public Diagram AddDiagram(String ID)
        {
            Diagram d = new Diagram(this,false, ID);
            return d;
        }

        /// <summary>
        /// Vrti referenciu na diagram uloen pod zadanm identifiktorom.
        /// </summary>
        /// <param name="ID">Identiciktor diagramu v rmci projektu.</param>
        /// <returns>Referencia na diagram</returns>
        public Diagram GetDiagram(String ID)
        {
            return (Diagram)GetChild(ID);
        }

        /// <summary>
        /// Kopruje vrchol aj s Template informciami.
        /// </summary>
        /// <returns>Vrti novovytvoren vrchol.</returns>
        public override TreeNode Clone()
        {
            Project copy = (Project)base.Clone();
            copy.nodeAiTemplates = nodeAiTemplates.Clone();
            // najdi zvolenu dopln. informaciu v kopii a zvol ju:
            if (BindingTime != null) {
                copy.bindingTime = copy.AiItems.GetAi(BindingTime.Key);
//                copy.bindingTime = copy.nodeAiTemplates.FeatureAiItems.GetAi(BindingTime.Key);
//                copy.bindingTime = copy.nodeAiTemplates.DiagramAiItems.GetAi(BindingTime.Key);
            }
            else copy.bindingTime = null;
            return (TreeNode)copy;
        }
    }
}
