using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace FM
{
    public partial class AiForm1 : Form
    {
        AllPossibleAiValues aPVProject ;
        AllPossibleAiValues aPVDiagram ;
        AllPossibleAiValues aPVFeature ;

        NodeAiTemplates nodAiTemplate;

        /// <summary>
        /// AiItem, na ktor sa viae BindingTime
        /// </summary>
        public AiItem BindingTimeAiItem;
        /// <summary>
        /// Pomocn prepna pre testovanie BindingTime
        /// </summary>
        public bool BTset;

        /// <summary>
        /// Vrti/nastav referenciu na ablnu dodatonch informci.
        /// </summary>
        public NodeAiTemplates NodeAiTemplates
        {
            get { return nodAiTemplate; }
            set { nodAiTemplate = value; }
        }
        /// <summary>
        /// Kontruktor formulra pre manaovanie ablny s dodatonmi informciami.
        /// </summary>
        /// <param name="aPVProject">Referencie na vetky mon Ai hodnoty pre projekt.</param>
        /// <param name="aPVDiagram">Referencie na vetky mon Ai hodnoty pre diagram.</param>
        /// <param name="aPVFeature">Referencie na vetky mon Ai hodnoty pre vlastnosti.</param>
        public AiForm1(AllPossibleAiValues aPVProject, AllPossibleAiValues aPVDiagram, AllPossibleAiValues aPVFeature)
        {
            InitializeComponent();

            this.aPVProject = aPVProject;
            this.aPVDiagram = aPVDiagram;
            this.aPVFeature = aPVFeature;

            BindingTimeAiItem = null;
            BTset = false;
        }

        private void AiForm1_Load(object sender, EventArgs e)
        {
            nodAiTemplate = new NodeAiTemplates();
        }

        private void addNewSelectableAiItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            String selNodName = treeView1.SelectedNode.Name;

            AiItems TmpAiItems = null;
            AllPossibleAiValues TmpAiAPV = null;

            switch (selNodName)
            {
                case "Project":
                    TmpAiItems = nodAiTemplate.ProjectAiItems;
                    TmpAiAPV = aPVProject;
                    break;

                case "Diagram":
                    TmpAiItems = nodAiTemplate.DiagramAiItems;
                    TmpAiAPV = aPVDiagram;
                    break;

                case "Feature":
                    TmpAiItems = nodAiTemplate.FeatureAiItems;
                    TmpAiAPV = aPVFeature;
                    break;
            }

            AiForm2 aiForm2 = new AiForm2(TmpAiAPV, treeView1.SelectedNode.Text, TmpAiItems, treeView1);

            if (selNodName == "Project")
                aiForm2.BTcheckBox.Enabled = true;
  
            aiForm2.DialogResult = aiForm2.ShowDialog();

            if (aiForm2.DialogResult == DialogResult.OK)
            {

                if (selNodName == "Project")
                {
                    if (aiForm2.BTcheckBox.Checked == true)
                    {
                        BindingTimeAiItem = TmpAiItems.GetAi(aiForm2.selectableAiName.Text);
                        BTset = true;

                    }
                    else if (aiForm2.BTcheckBox.Checked == false)
                    {
                        BindingTimeAiItem = null;
                        BTset = false;

                    }
                }
            }

            if (aiForm2.DialogResult == DialogResult.Cancel)
            {
                if (TmpAiItems.GetAi(aiForm2.selectableAiName.Text) != null)
                    TmpAiItems.RemoveAi(aiForm2.selectableAiName.Text);
            }
        }

        private void addNewTextualAiItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AiForm3 aiForm3 = new AiForm3();
            if (aiForm3.ShowDialog() == DialogResult.OK)
            {
                if (aiForm3.textualAIName.Text.Equals(""))
                    return;

                AiItems TmpAiItems = null;
                AllPossibleAiValues TmpAiAPV = null;

                switch (treeView1.SelectedNode.Name)
                {
                    case "Project":
                        TmpAiItems = nodAiTemplate.ProjectAiItems;
                        TmpAiAPV = aPVProject;
                        break;

                    case "Diagram":
                        TmpAiItems = nodAiTemplate.DiagramAiItems;
                        TmpAiAPV = aPVDiagram;
                        break;

                    case "Feature":
                        TmpAiItems = nodAiTemplate.FeatureAiItems;
                        TmpAiAPV = aPVFeature;
                        break;
                }

                if(TmpAiItems.GetAi(aiForm3.textualAIName.Text) != null)
                    return;
                /*
                if ((aiForm3.textualAIName.Text == "BindingTime") || (aiForm3.textualAIName.Text == "InstantiationTime"))
                {
                    MessageBox.Show("AiItem name was reserved! Choose another.");
                    return;
                }
                */
                TmpAiItems.AddAi(aiForm3.textualAIName.Text, false, TmpAiAPV);
                TmpAiItems.GetAi(aiForm3.textualAIName.Text).SetAi(aiForm3.textualAIValue.Text);
                
                // Nastavenie popisu AiItem
                TmpAiItems.GetAi(aiForm3.textualAIName.Text).Description = aiForm3.textualAiDescription.Text;



                this.treeView1.SelectedNode.Nodes.Add(aiForm3.textualAIName.Text);
            }
                
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if ((treeView1.SelectedNode.Parent != null) &&
                ((treeView1.SelectedNode.Parent.Name == "Project") ||
                 (treeView1.SelectedNode.Parent.Name == "Diagram") ||
                 (treeView1.SelectedNode.Parent.Name == "Feature")))
            {
                this.ContextMenuStrip = contextMenuStrip2;
            }
            else
                this.ContextMenuStrip = null;      
        }

        private void treeView1_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            AiForm3 aiForm3 = new AiForm3();

            if (treeView1.SelectedNode.Parent == null)
                return;

            AiItems TmpAiItems = null;

            switch (treeView1.SelectedNode.Parent.Name)
            {
                case "Project":
                    TmpAiItems = nodAiTemplate.ProjectAiItems;
                    break;

                case "Diagram":
                    TmpAiItems = nodAiTemplate.DiagramAiItems;
                    break;

                case "Feature":
                    TmpAiItems = nodAiTemplate.FeatureAiItems;
                    break;
            }

            aiForm3.textualAIName.Text = TmpAiItems.GetAi(treeView1.SelectedNode.Text).Key;
            aiForm3.textualAIValue.Text = TmpAiItems.GetAi(treeView1.SelectedNode.Text).CurrentValue;
            aiForm3.textualAiDescription.Text = TmpAiItems.GetAi(treeView1.SelectedNode.Text).Description;

            aiForm3.Text = "";
            
            aiForm3.textualAIName.ReadOnly = true;
            aiForm3.textualAiDescription.ReadOnly = true;
            aiForm3.textualAIValue.ReadOnly = true;
            
            aiForm3.ShowDialog();
            
        }

        private void changeAiItem_Click(object sender, EventArgs e)
        {
            String selNodName = treeView1.SelectedNode.Text;
            
            //if (treeView1.SelectedNode.Parent == null)
            //    return;

            AiItems TmpAiItems = null;
            AllPossibleAiValues TmpAiAPV = null;

            switch(treeView1.SelectedNode.Parent.Name)
            {
                case "Project":
                    TmpAiItems = nodAiTemplate.ProjectAiItems;
                    TmpAiAPV = aPVProject;
                    break;
                    
                case "Diagram":
                    TmpAiItems = nodAiTemplate.DiagramAiItems;
                    TmpAiAPV = aPVDiagram;
                    break;

                case "Feature":
                    TmpAiItems = nodAiTemplate.FeatureAiItems;
                    TmpAiAPV = aPVFeature;
                    break;
            }

            if (TmpAiItems.GetAi(selNodName).IsSelectable())
            {
                AiForm2 aiForm2 = new AiForm2(TmpAiAPV, treeView1.SelectedNode.Parent.Text, TmpAiItems, treeView1);
                aiForm2.Text = "Ai Item modification";
                aiForm2.selectableAiName.Text = selNodName;        
                aiForm2.selectableAiName.ReadOnly = true;
                aiForm2.selectableAiDescription.Text = TmpAiItems.GetAi(selNodName).Description;
                

                //selectableAiCV.Text = TmpAiItems.GetAi(selNodName).CurrentValue.ToString();

                for (int i = 0; i < TmpAiAPV.GetReferenceToKey(selNodName).PossibleValuesCount(); i++)
                {
                    aiForm2.selectableAiPV.Items.Add(TmpAiAPV.GetReferenceToKey(selNodName).PossibleValues[i]);
                }

                if (treeView1.SelectedNode.Parent.Text == "Project")
                    aiForm2.BTcheckBox.Enabled = true;

                if ((BindingTimeAiItem != null) && (BTset == true))
                    if (selNodName == BindingTimeAiItem.Key)
                        aiForm2.BTcheckBox.Checked = true;

                // Hodnota na zaciatku checkboxu
                bool bStart = aiForm2.BTcheckBox.Checked;
                

                if (aiForm2.ShowDialog() == DialogResult.OK)
                {
                    //this.treeView1.SelectedNode.Nodes.Add(aiForm2.selectableAiName.Text);

                    // Hodnota na konci checkboxu
                    bool bStop = aiForm2.BTcheckBox.Checked;

                    // Ak nastala zmena v checkboxe
                    if (bStart != bStop)
                    {
                        if (aiForm2.BTcheckBox.Checked == true)
                        {
                            BindingTimeAiItem = TmpAiItems.GetAi(aiForm2.selectableAiName.Text);
                            BTset = true;

                        }
                        else if (aiForm2.BTcheckBox.Checked == false)
                        {
                            BindingTimeAiItem = null;
                            BTset = false;

                        }
                    }
                }

            }
            else
            {
                AiForm3 aiForm3 = new AiForm3();
                aiForm3.Text = "Ai Item modification";
                aiForm3.textualAIName.Text = selNodName;
                aiForm3.textualAIName.ReadOnly = true;
                aiForm3.textualAIValue.Text = TmpAiItems.GetAi(selNodName).CurrentValue.ToString();
                aiForm3.textualAiDescription.Text = TmpAiItems.GetAi(selNodName).Description;

                if (aiForm3.ShowDialog() == DialogResult.OK)
                {
                    TmpAiItems.GetAi(aiForm3.textualAIName.Text).SetAi(aiForm3.textualAIValue.Text);
                    TmpAiItems.GetAi(aiForm3.textualAIName.Text).Description = aiForm3.textualAiDescription.Text;
                }
            }
        }

        private void removeAiItem_Click(object sender, EventArgs e)
        {
            String selNodName = treeView1.SelectedNode.Text;

            //if (treeView1.SelectedNode.Parent == null)
            //    return;

            AiItems TmpAiItems = null;
            AllPossibleAiValues TmpAiAPV = null;

            switch (treeView1.SelectedNode.Parent.Name)
            {
                case "Project":
                    TmpAiItems = nodAiTemplate.ProjectAiItems;
                    TmpAiAPV = aPVProject;
                    break;

                case "Diagram":
                    TmpAiItems = nodAiTemplate.DiagramAiItems;
                    TmpAiAPV = aPVDiagram;
                    break;

                case "Feature":
                    TmpAiItems = nodAiTemplate.FeatureAiItems;
                    TmpAiAPV = aPVFeature;
                    break;
            }

            if (TmpAiItems.GetAi(selNodName) != null)
            {
                if (BindingTimeAiItem.Key == selNodName)
                {
                    BindingTimeAiItem = null;
                    BTset = false;
                }
                TmpAiItems.RemoveAi(selNodName);
                treeView1.SelectedNode.Nodes.Remove(treeView1.SelectedNode);
            }
        }
        /// <summary>
        /// Funkcia validuje vytvoren ablnu.
        /// </summary>
        /// <returns></returns>
        public bool ValidateTemplate()
        {
            if((BindingTimeAiItem == null) || (BTset == false))
                return false;
            else
                return true;
        }

        private void AiForm1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Ak pouzivatel nevytvoril AiItem template spravne (napr. chyba BindingTime), tak tu sa to opravi.
            if (!ValidateTemplate())
            {

                //TmpAiItems = nodAiTemplate.ProjectAiItems;
                //TmpAiAPV = aPVProject;

                // Treba vytvorit jednu AiItem "BindingTime" pre project s jedinou PV "sourceTime".
                nodAiTemplate.ProjectAiItems.AddAi("BindingTime", true, aPVProject);
                nodAiTemplate.ProjectAiItems.GetAi("BindingTime").AddValueToSet("sourceTime");
                nodAiTemplate.ProjectAiItems.GetAi("BindingTime").SetAi("sourceTime");

                BindingTimeAiItem = nodAiTemplate.ProjectAiItems.GetAi("BindingTime");
                BTset = true;

                // Taku istu AiItem treba vytvorit pre feature.
                nodAiTemplate.FeatureAiItems.AddAi("BindingTime", true, aPVFeature);
                nodAiTemplate.FeatureAiItems.GetAi("BindingTime").AddValueToSet("sourceTime");
                nodAiTemplate.FeatureAiItems.GetAi("BindingTime").SetAi("sourceTime");

                nodAiTemplate.DiagramAiItems.AddAi("BindingTime", true, aPVDiagram);
                nodAiTemplate.DiagramAiItems.GetAi("BindingTime").AddValueToSet("sourceTime");
                nodAiTemplate.DiagramAiItems.GetAi("BindingTime").SetAi("sourceTime");

                /* Toto zatial netreba
                // A referenciu na nu pridat do templatu pre diagram.
                AiItem refAiItem = nodAiTemplate.FeatureAiItems.GetAi("BindingTime");
                nodAiTemplate.DiagramAiItems.AddAiByReference(refAiItem);
                */

                // we know that:
                // MessageBox.Show("Binding time in the project wasn't set correctly! The error was automatically fixed.");

            }
            else
            {
                /* POVODNA IDEA 
                // Taku istu AiItem treba vytvorit pre feature.
                nodAiTemplate.FeatureAiItems.AddAi("BindingTime", true, aPVFeature);
                nodAiTemplate.DiagramAiItems.AddAi("InstantiationTime", true, aPVDiagram);

                PossibleAiValues pV = aPVProject.GetReferenceToKey(BindingTimeAiItem.Key);

                for (int i = 0; i < pV.PossibleValues.Count; i++)
                {
                    nodAiTemplate.FeatureAiItems.GetAi("BindingTime").AddValueToSet(pV.PossibleValues[i].ToString());
                    nodAiTemplate.DiagramAiItems.GetAi("InstantiationTime").AddValueToSet(pV.PossibleValues[i].ToString());
                }
 
                nodAiTemplate.FeatureAiItems.GetAi("BindingTime").SetAi(BindingTimeAiItem.CurrentValue);
                */
                   
                nodAiTemplate.FeatureAiItems.AddAi(BindingTimeAiItem.Key, true, aPVFeature);
                nodAiTemplate.DiagramAiItems.AddAi(BindingTimeAiItem.Key, true, aPVDiagram);

                PossibleAiValues pV = aPVProject.GetReferenceToKey(BindingTimeAiItem.Key);

                for (int i = 0; i < pV.PossibleValues.Count; i++)
                {
                    nodAiTemplate.FeatureAiItems.GetAi(BindingTimeAiItem.Key).AddValueToSet(pV.PossibleValues[i].ToString());
                    nodAiTemplate.DiagramAiItems.GetAi(BindingTimeAiItem.Key).AddValueToSet(pV.PossibleValues[i].ToString());
                }

                if (BindingTimeAiItem.CurrentValue.ToString() == "")
                {
                    nodAiTemplate.ProjectAiItems.GetAi(BindingTimeAiItem.Key).SetAi(pV.PossibleValues[0].ToString());
                    nodAiTemplate.DiagramAiItems.GetAi(BindingTimeAiItem.Key).SetAi(pV.PossibleValues[0].ToString());
                    nodAiTemplate.FeatureAiItems.GetAi(BindingTimeAiItem.Key).SetAi(pV.PossibleValues[0].ToString());
                }
                else
                {
                    nodAiTemplate.DiagramAiItems.GetAi(BindingTimeAiItem.Key).SetAi(pV.PossibleValues[0].ToString());
                    nodAiTemplate.FeatureAiItems.GetAi(BindingTimeAiItem.Key).SetAi(pV.PossibleValues[0].ToString());
                }

                nodAiTemplate.DiagramAiItems.GetAi(BindingTimeAiItem.Key).Description = BindingTimeAiItem.Description;
                nodAiTemplate.FeatureAiItems.GetAi(BindingTimeAiItem.Key).Description = BindingTimeAiItem.Description;

            }
        }      
    }
}