using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.IO;

namespace FM.forms
{
    /// <summary>
    /// Trieda ConstraintsMainForm predstavuje skupinu operacii s grafickym rozhranim constraints managera.
    /// </summary>
    public partial class ConstraintsMainForm : Form
    {
        private Diagram diagram;
        private bool status = false;
        private int changed_idnex;

        /// <summary>
        /// Kontruktor obrazovky ConstraintsMainForm 
        /// </summary>
        /// <param name="diagram">Vstupnm parametrom je diagram na ktor sa viau zobrazovan constrainty.</param>
        public ConstraintsMainForm(Diagram diagram)
        {
            InitializeComponent();
            this.diagram = diagram;

            foreach (Expression exp in diagram.Constraints.GetAllConstraints())
            {
                //                listBox1.Items.Add(new ListItem(exp.GetID, exp.GetExpression));
                listBox1.Items.Add(exp.exp);
            }

        }

        /// <summary>
        /// Metda kontextovho menu pre editciu obmedzeni v listbox1.
        /// </summary>
        private void modifyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex != -1)
            {
                status = true;
                textBox1.Text = (String)listBox1.SelectedItem;
                changed_idnex = listBox1.SelectedIndex;
                listBox1.Enabled = false;
                textBox1.Focus();
            }
        }

        /// <summary>
        /// Metda kontextovho menu pre mazanie obmedzen v listbox1.
        /// </summary>
        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            remove_item();       
        }

        /// <summary>
        /// Tlaidlo potvrdzuje zaradenie obmedzenia medzi constraints.
        /// </summary>
        private void button1_Click(object sender, EventArgs e)
        {
            confirm();    
        }


        /// <summary>
        /// Metda na mazanie poloiek obmedzen z listbox1.
        /// </summary>
        private void remove_item()
        {
            if (listBox1.SelectedIndex != -1)
            {
                diagram.Constraints.RemoveConstraint((String)listBox1.SelectedItem);
                listBox1.Items.Remove(listBox1.SelectedItem);
            }
        }


        /// <summary>
        /// Metda ktor zapisuje vraz z texbox1 do obmedzen. Poda stavu rozhoduje o pridan, alebo nahraden obmedzenia.
        /// </summary>
        private void confirm()
        {
            if (status)
            {
                if (textBox1.Text != "")
                {
                    if (diagram.Constraints.GetAllConstraints().Count != 0)
                    {
                        diagram.Constraints.RemoveConstraint((String)listBox1.Items[changed_idnex]);
                        diagram.Constraints.AddConstraint(textBox1.Text);
                        listBox1.Items.RemoveAt(changed_idnex);
                        listBox1.Items.Insert(changed_idnex, textBox1.Text);
                    }
                    else
                    {
                        diagram.Constraints.AddConstraint(textBox1.Text);
                        listBox1.Items.Add(textBox1.Text);
                        textBox1.Clear();                    
                    }
                    
                }
                listBox1.Enabled = true;
                status = false;
                textBox1.Clear();
                listBox1.Refresh();
                listBox1.Focus();

            }
            else
            {
                if (textBox1.Text != "")
                {
                    diagram.Constraints.AddConstraint(textBox1.Text);
                    listBox1.Items.Add(textBox1.Text);
                    textBox1.Clear();
                }
            }
        }


        /// <summary>
        /// Metda ktor kontoluje akcie na zklade stlania klvesy v textboxe.
        /// </summary>
        private void textBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == System.Windows.Forms.Keys.Escape)
            {
                if (status)
                {
                    listBox1.Enabled = true;
                    status = false;
                    textBox1.Clear();
                    listBox1.Refresh();
                    listBox1.Focus();

                }
                else
                {
                    if (textBox1.Text == "")
                    {
                        this.Close();
                    }
                    else
                    {
                        textBox1.Clear();
                    }
                }
            }

            if (e.KeyCode == System.Windows.Forms.Keys.Enter)
            {
                confirm();
            }      
        }
        /*
        private void button2_Click(object sender, EventArgs e)
        {
            diagram.Constraints.ValidateAll(diagram);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex != -1)
            {
                Expression exp = new Expression(1, "", "-", "&&", "||", "<=>", "=>");
                MessageBox.Show(exp.EvaluateParsedExpression((String)listBox1.SelectedItem));
            }
        }
        */

        /// <summary>
        /// Metda, ktor upravuje vraz pri dvojkliku na listbox1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex != -1)
            {
                status = true;
                textBox1.Text = (String)listBox1.SelectedItem;
                changed_idnex = listBox1.SelectedIndex;
                listBox1.Enabled = false;
                textBox1.Focus();
            }
        }

        /// <summary>
        /// Metda, ktor zmae vraz z listbox1 po stlaen tlaidla delete.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == System.Windows.Forms.Keys.Delete)
            {
                this.remove_item();
            }
        }

        /*
        private void button4_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex != -1)
            {
                Expression exp = new Expression(1, (String)listBox1.SelectedItem, "-", "&&", "||", "<=>", "=>");
                MessageBox.Show(exp.ParseExpression(diagram));
            }
        }
         */


        /// <summary>
        /// Metda volan pri zmene vrazu z inch obrazoviek.
        /// </summary>
        /// <param name="s"></param>
        public void ChangeConstraint(String s)
        {
            if (s != "")
            {
                status = true;
                textBox1.Text = s;
                changed_idnex = listBox1.Items.IndexOf(s);
                listBox1.Enabled = false;
                textBox1.Focus();
            }
            
        }


    }
}