using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace FM
{
    /// <summary>
    /// Formulr na nastavenie vlastnost feature a groupy (nastavuje meno a kardinalitu,
    /// priom meno sa d nastavi len pri vytvran, pri editci je u read-only).
    /// </summary>
    public partial class PropertiesForm : Form
    {
        /// <summary>
        /// Odkaz na uzol, pod ktor pridvame, alebo pod ktorm editujeme.
        /// </summary>
        private DNode parentNode;
		/// <summary>
		/// Odkaz na uzol, ktory editujeme
		/// </summary>
		private DNode currentNode;
        /// <summary>
        /// Flag, i ide o feature alebo groupu.
        /// </summary>
        private bool isFeature;

		/// <summary>
		/// Minimlna kardnialita
		/// </summary>
		public int cardMax;
		/// <summary>
		/// Maximlna kardinalita
		/// </summary>
		public int cardMin;

        /// <summary>
        /// Nepouvan kontruktor
        /// </summary>
        public PropertiesForm()
        {
            InitializeComponent();
            parentNode = null;
            isFeature = false;

            this.Text = "Until it sleeps";
        }

        /// <summary>
        /// Kontruktor nastav sprvne titulky prednastav hotnoty - ide o pridvanie
        /// </summary>
        /// <param name="parentNode">Odkaz na uzol, pod ktor pridvame, alebo ktor editujeme</param>
        /// <param name="isFeature">Flag, i ide o feature alebo groupu</param>
        public PropertiesForm(DNode parentNode, bool isFeature)
        {
            InitializeComponent();
			this.currentNode = null;
            this.parentNode = parentNode;
            this.isFeature = isFeature;

            this.featureSelected.Visible = false;

            if (isFeature)
            {
                this.Text = "Add new feature";
            }
            else
            {
                this.Text = "Add new group";
				this.name.Visible = false;
				this.prompt.Visible = false;
			}
        }


        /// <summary>
        /// Kontruktor nastav sprvne titulky prednastav hotnoty a zake meni
        /// meno - editujeme.
        /// </summary>
        /// <param name="currentNode">Odkaz na uzol, pod ktor pridvame, alebo ktor editujeme</param>
        /// <param name="name">Aktulne meno</param>
        /// <param name="cardinalityMin">Aktulna minimlna kardinalita</param>
        /// <param name="cardinalityMax">Aktulna maximlna kardinalita</param>
        /// <param name="isFeature">Flag, i ide o feature alebo groupu</param>
        public PropertiesForm(DNode currentNode, string name, int cardinalityMin, int cardinalityMax, bool isFeature)
        {
            InitializeComponent();

            this.name.Text = name;
			if (currentNode.GetType() == typeof(Diagram))
			{
				if (((Diagram)currentNode).IsInstance || ((Diagram)currentNode).HasInstance)
				{
		            this.name.ReadOnly = true;
				}
			}
			else
			{
				if (((Diagram)currentNode.Diagram).IsInstance || ((Diagram)currentNode.Diagram).HasInstance)
				{
					this.name.ReadOnly = true;
				}
			}

            this.cardinalityMin.Value = cardinalityMin;
            if (cardinalityMax == -1)
            {
                this.cardinalityMax.Value = 1;
                this.cardinalityMax.Enabled = false;
                this.infinite.Checked = true;
            }
            else {
                this.cardinalityMax.Value = cardinalityMax;
            }

			this.currentNode = currentNode;
            this.parentNode = (DNode)currentNode.Parent;
            this.isFeature = isFeature;

            this.featureSelected.Visible = false;

            if (isFeature)
            {
                this.Text = "Feature properties";

                DFeature tmp = (DFeature)this.currentNode;
                if (tmp.Diagram.IsInstance)
                {
                    this.featureSelected.Visible = true;
                    this.featureSelected.Checked = tmp.Selected;
                }
            }
            else
            {
                this.Text = "Group properties";
				this.name.Visible = false;
				this.prompt.Visible = false;
            }

            if (this.currentNode.Diagram.IsInstance || this.currentNode.Diagram.HasInstance)
            {
                this.cardinalityMin.Enabled = false;
                this.cardinalityMax.Enabled = false;
				this.infinite.Enabled = false;
            }
        }   

        /// <summary>
        /// Metda over, i s zadan hodnoty prpustn, ak nie s nedovol sa sumbitn formulr.
        /// </summary>
        /// <returns>Pravda ak je vetko v poriadku</returns>
        public bool ValidateInput()
        {
			if (this.isFeature && this.name.Text.Equals(""))
            {
                MessageBox.Show("Please fill in the name", "No name set", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				this.name.Focus();
                return false;
            }

            if (this.isFeature && this.name.Enabled == true)
            {
				if (currentNode == null || (currentNode != null && currentNode.Name != this.name.Text))
				{
					if (parentNode.GetChild(this.name.Text) != null)
					{
						MessageBox.Show("Feature or group with this name already exists in this branch. Please pick some another name.", "Name is not unique", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
						return false;
					}

					if (this.isFeature && parentNode.GetChildFeature(this.name.Text) != null)
					{
						MessageBox.Show("Feature with this name already exists in this branch. Please pick some another name.", "Name is not unique", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
						return false;
					}
				}
            }

            this.cardMin = (int)this.cardinalityMin.Value;
			if (this.infinite.Checked)
			{
				this.cardMax = -1;
			}
			else
			{
				this.cardMax = (int)this.cardinalityMax.Value;
			}

			if (this.cardMax != -1 && this.cardMax < this.cardMin)
            {
                MessageBox.Show("Maximal cardinality must be greater or equal comparing to the minimal cardinality.", "Cardinality problem", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

			return true;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.ValidateInput())
            {
                this.DialogResult = DialogResult.OK;
            }
        }

		private void PropertiesForm_Load(object sender, EventArgs e)
		{
			//this.name.Focus();
		}

		private void PropertiesForm_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar == (char)Keys.Enter)
			{
				this.button1_Click(null, null);
			}
			else if (e.KeyChar == (char)Keys.Escape)
			{
				this.DialogResult = DialogResult.Cancel;
			}
		}

		private void infinite_CheckedChanged(object sender, EventArgs e)
		{
			if (this.infinite.Checked)
			{
				this.cardinalityMax.Enabled = false;
			}
			else
			{
				this.cardinalityMax.Enabled = true;
			}
		}
    }
}